/*
 *  dialog_windows.cpp - Win32 dialog boxes
 *
 *  Basilisk II (C) 1997-1999 Christian Bauer
 *
 *  Ported to Windows by Lauri Pesonen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <process.h>
#include <windowsx.h>
#include "sysdeps.h"
#include "prefs.h"
#include "main_windows.h"
#include "dialog_windows.h"
#include "util_windows.h"
#include "user_strings.h"
#include "version.h"
#include "resource.h"

#define MB(x) (1048576L*(x))

typedef struct {
	char name[_MAX_PATH];
	uint32 size;
} NEWHFVPROCPARAMS, *PNEWHFVPROCPARAMS;

int16 FAR PASCAL NEWHFVPROC( HWND hDlg, unsigned message, WPARAM wParam, LPARAM lParam )
{
	int result = 0;
	static PNEWHFVPROCPARAMS params;

	switch (message) {
		case WM_INITDIALOG:
			params = (PNEWHFVPROCPARAMS)lParam;
			SetDlgItemText( hDlg, IDC_MAKEHFV_SIZE, "10" );
			SetDlgItemText( hDlg, IDC_MAKEHFV_NAME, "New.HFV" );
			result = 1;
			break;
		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam,lParam)) {
				case IDCANCEL:
					EndDialog( hDlg, 0 );
					result = 1;
					break;
				case IDOK:
					char buf[100];
					GetDlgItemText( hDlg, IDC_MAKEHFV_SIZE, buf, sizeof(buf) );
					params->size = MB(atoi(buf));
					GetDlgItemText( hDlg, IDC_MAKEHFV_NAME, params->name, _MAX_PATH );
					if(exists(params->name)) {
						MessageBox(hDlg,"There is already a file with this name.","",MB_OK|MB_ICONSTOP);
					} else {
						if(params->size < MB(1)) {
							params->size = MB(1);
							MessageBox(hDlg,"The given size was too small -- adjusted to 1MB.","",MB_OK|MB_ICONSTOP);
						} else if(params->size > MB(2000)) {
							params->size = MB(2000);
							MessageBox(hDlg,"The given size was too large -- adjusted to 2000 MB.","",MB_OK|MB_ICONSTOP);
						}
						EndDialog( hDlg, 1 );
					}
					result = 1;
					break;
			}
			break;
	  default:
			result = 0;
	}
  return(result);
}

void new_volume( HWND hWnd )
{
	NEWHFVPROCPARAMS params;

	memset( &params, 0, sizeof(params) );

	if(DialogBoxParam( (HINSTANCE)hInst, MAKEINTRESOURCE(IDD_NEW_HFV), hWnd, (DLGPROC)NEWHFVPROC, (LPARAM)&params )) {
		if(create_file( params.name, params.size )) {
			int answer = MessageBox(
				hWnd,
				"The file was created. Do you want to update the preferences file now\n"
				"(if there are comment lines they will be lost) ?",
				"",
				MB_YESNO|MB_ICONQUESTION
			);
			if(answer == IDYES) {
				PrefsAddString("disk", params.name);
				SavePrefs();
				MessageBox(
					hWnd,
					"The file will be mounted next time you start Basilisk II.\nMacOS will report it as unreadable and prompt to format it.",
					"",
					MB_OK|MB_ICONINFORMATION
				);
			} else {
				char tmp[512];
				sprintf( 
					tmp, 
					"Insert the line \"disk %s\" to you preferences file. Restart Basilisk II. MacOS will report the volume as unreadable and prompt to format it.",
					params.name
				);
				MessageBox(
					hWnd,
					tmp,
					"",
					MB_OK|MB_ICONINFORMATION
				);
			}
		}	else {
			MessageBox(hWnd,"Could not create the file. Make sure that there is enough free space, and that you don't already have a file with this name.","",MB_OK|MB_ICONSTOP);
		}
	}
}

int16 FAR PASCAL ABOUTPROC( HWND hDlg, unsigned message, WPARAM wParam, LPARAM lParam )
{
	int result = 0;
	char about[100];

	switch (message) {
		case WM_INITDIALOG:
			wsprintf( about, GetString(STR_ABOUT_TEXT1), VERSION_MAJOR, VERSION_MINOR );
			wsprintf( &about[strlen(about)], " %s", GetString(STR_ABOUT_TEXT2) );
			SetWindowText( GetDlgItem(hDlg,IDC_ABOUT_VERSION), about );
			result = 1;
			break;
		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam,lParam)) {
				case IDOK:
					EndDialog( hDlg, 1 );
					result = 1;
					break;
			}
			break;
	  default:
			result = 0;
	}
  return(result);
}

void about( HWND hWnd )
{
	DialogBox( (HINSTANCE)hInst, MAKEINTRESOURCE(IDD_ABOUT), hWnd, (DLGPROC)ABOUTPROC );
}
