/*
 *  main_windows.cpp - Startup code for Win32
 *
 *  Basilisk II (C) 1997-1999 Christian Bauer
 *
 *  Ported to Windows by Lauri Pesonen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// TODO: version info must be visible somewhere
//       rom file name prefs item

#include <stdio.h>
#include <process.h>

#include "sysdeps.h"
#include "readcpu.h"
#include "memory.h"
#include "newcpu.h"
#include "sys.h"
#include "xpram.h"
#include "timer.h"
#include "sony.h"
#include "disk.h"
#include "cdrom.h"
#include "scsi.h"
#include "video.h"
#include "serial.h"
#include "ether.h"
#include "clip.h"
#include "rom_patches.h"
#include "prefs.h"
#include "user_strings.h"
#include "version.h"
#include "main.h"
#include "adb.h"
#include "cpu_emulation.h"
#include "main_windows.h"
#include "video_windows.h"
#include "timer_windows.h"
#include "xpram_windows.h"
#include "desktop_windows.h"
#include "sys_windows.h"
#include "prefs_windows.h"
#include "dialog_windows.h"
#include "resource.h"

#include <dbt.h>
#include <windowsx.h>

#define DEBUG 0
#include "debug.h"

// Constants
const char ROM_FILE_NAME[] = "ROM";

// RAM and ROM pointers
// uint32 RAMBaseMac;		// RAM base (Mac address space)
// uint8 *RAMBaseHost;		// RAM base (host address space)
// uint32 RAMSize;				// Size of RAM

// uint32 ROMBaseMac;		// ROM base (Mac address space)
// uint8 *ROMBaseHost;		// ROM base (host address space)
// uint32 ROMSize;				// Size of ROM

// CPU and FPU type
int CPUType;
bool CPUIs68060;
int FPUType;

uint32 InterruptFlags = 0;
static CRITICAL_SECTION intflag_csection;

// os flag set up in initialize()
int win_os; 

// TRUE if older than OSR2
BOOL win_os_old = FALSE; 

BOOL mem_8_only = FALSE;

HWND hMainWnd = 0;
HWND hAuxWnd = 0;
HINSTANCE hInst = 0;

// mouse and keyboard
static bool capturing = false;
static bool dragging = false;
static bool tracking_menu = false;

// Keep track what Mac keys are down so we can make them up when we lose focus.
#define MAX_KEYS 256
static BYTE mac_keys_down[MAX_KEYS];

// Floppy detection
UINT registered_floppy_hotkey = VK_F11;

// Mac time starts in 1904, Unix time in 1970, this is the offset in seconds
const uint32 TIME_OFFSET = 0x7c25b080;

// Some thread handles
HANDLE GUIThread_handle = 0;
HANDLE CPUThread_handle = 0;
HANDLE TickThread_handle = 0;
HANDLE OneSecThread_handle = 0;
HANDLE PramThread_handle = 0;

// local forwards
unsigned int WINAPI cpu_thread(LPVOID param);
unsigned int WINAPI tick_func(LPVOID param);
unsigned int WINAPI one_sec_func(LPVOID param);
unsigned int WINAPI old_dt_func(LPVOID param);

#define DOWN_MASK 0x8000
// #define DOWN_MASK 0x80
#define TOGGLE_MASK 1

SHORT my_get_key_state( int vKey )
{
	// BYTE state[256];

	// Win95 version of GetAsyncKeyState() does not distinguish between left and right,
	// but it seems to be that even GetKeyboardState() has problems.
	// return(GetAsyncKeyState(vKey));

	return(GetKeyState(vKey));

	// GetKeyboardState( state );
	// return( state[vKey] );
}

// TODO: use these as default values, read the table from a file
static int key_win2mac( WPARAM wparam, bool down )
{
	switch (wparam) {
    case 'A': return 0x00;
    case 'B': return 0x0b;
    case 'C': return 0x08;
    case 'D': return 0x02;
    case 'E': return 0x0e;
    case 'F': return 0x03;
    case 'G': return 0x05;
    case 'H': return 0x04;
    case 'I': return 0x22;
    case 'J': return 0x26;
    case 'K': return 0x28;
    case 'L': return 0x25;
    case 'M': return 0x2e;
    case 'N': return 0x2d;
    case 'O': return 0x1f;
    case 'P': return 0x23;
    case 'Q': return 0x0c;
    case 'R': return 0x0f;
    case 'S': return 0x01;
    case 'T': return 0x11;
    case 'U': return 0x20;
    case 'V': return 0x09;
    case 'W': return 0x0d;
    case 'X': return 0x07;
    case 'Y': return 0x10;
    case 'Z': return 0x06;

		case '1': return 0x12;
		case '2': return 0x13;
		case '3': return 0x14;
		case '4': return 0x15;
		case '5': return 0x17;
		case '6': return 0x16;
		case '7': return 0x1a;
		case '8': return 0x1c;
		case '9': return 0x19;
		case '0': return 0x1d;

		case VK_UP: return 0x3e;
		case VK_DOWN: return 0x3d;
		case VK_LEFT: return 0x3b;
		case VK_RIGHT: return 0x3c;

		case VK_PRIOR: return 0x74;
		case VK_NEXT: return 0x79;

		case VK_DELETE: return 0x75;
		case VK_INSERT: return 0x72;
		case VK_HOME: return 0x73;
		case VK_END: return 0x77;

		// Is it really the case that these keys don't have proper VK_* values?
		case 0xBA: return 0x1e; //  ^
		case 0xDB: return 0x18; //  `
		case 0xBB: return 0x1b; // + ?
		case 0xBC: return 0x2b; // , ;
		case 0xBD: return 0x2c; // - _
		case 0xBE: return 0x2f; // . :
		case 0xBF: return 0x2a; // ' *
		case 0xC0: return 0x29; //  
		case 0xDC: return 0xa;  //  
		case 0xDD: return 0x21; //  
		case 0xDE: return 0x27; //  
		case 0xE2: return 0x32; // < > |
		
		case VK_TAB: return 0x30;
		case VK_RETURN: return 0x24;
		case VK_SPACE: return 0x31;
		case VK_BACK: return 0x33;
		case VK_ESCAPE: return 0x35;



		// Modifier keys

		case VK_SHIFT: return 0x38;

		case VK_CONTROL: 
			// Right control maps to option
			// Don't know which one of the controls was pressed/released, find out
			bool r_is_down, l_is_down, r_was_down, l_was_down;
			l_is_down = (my_get_key_state(VK_LCONTROL) & DOWN_MASK) != 0;
			r_is_down = (my_get_key_state(VK_RCONTROL) & DOWN_MASK) != 0;
			l_was_down = mac_keys_down[0x36] != 0;
			r_was_down = mac_keys_down[0x3A] != 0;

			if(l_is_down != l_was_down) {
				return 0x36; // Left Control. Think Reference says 0x3B which is wrong
			}
			if(r_is_down != r_was_down) {
				return 0x3A; // Right Control -> option
			}
			return 0x36; // Left Control.
			// return(-1);

		// Command
		case VK_MENU: return 0x37; 

		// option
		case VK_LWIN: return 0x3a;
		case VK_RWIN: return 0x3a;



		case VK_F1: return 0x7a;
		case VK_F2: return 0x78;
		case VK_F3: return 0x63;
		case VK_F4: return 0x76;
		case VK_F5: return 0x60;
		case VK_F6: return 0x61;
		case VK_F7: return 0x62;
		case VK_F8: return 0x64;
		case VK_F9: return 0x65;
		case VK_F10: return 0x6d;
		case VK_F11: return 0x67;
		case VK_F12: return 0x6f;

		case VK_PRINT: return 0x69;
		case VK_SNAPSHOT: return 0x69;

		case VK_PAUSE: return 0x7f;  // Power key

		case VK_NUMPAD0: return 0x52;
		case VK_NUMPAD1: return 0x53;
		case VK_NUMPAD2: return 0x54;
		case VK_NUMPAD3: return 0x55;
		case VK_NUMPAD4: return 0x56;
		case VK_NUMPAD5: return 0x57;
		case VK_NUMPAD6: return 0x58;
		case VK_NUMPAD7: return 0x59;
		case VK_NUMPAD8: return 0x5b;
		case VK_NUMPAD9: return 0x5c;

		case VK_DECIMAL: return 0x41;
		case VK_ADD: return 0x45;
		case VK_SUBTRACT: return 0x4e;
		case VK_MULTIPLY: return 0x43;
		case VK_DIVIDE: return 0x4b;
		case VK_SEPARATOR: return 0x4c;



		// Toggle keys

		// Caps lock
		case VK_CAPITAL: 
			if(my_get_key_state(VK_CAPITAL) & TOGGLE_MASK) {
				if(down) {
					return 0x39;
				} else {
					// something wrong, -1 will make it in synch again
					return -1;
				}
			} else {
				if(down) {
					// something wrong, -1 will make it in synch again
					return -1;
				} else {
					return 0x39;
				}
			}
			// not reached
			break;

		// num lock, needs some work
		case 0x90: 
			return 0x47; // clear

		// scroll lock, come up with something here some day
		case 0x91: 
			/* MS DEV has this to say:
				Because the SetKeyboardState function alters the input state of the calling 
				thread and not the global input state of the system, an application cannot 
				use SetKeyboardState to set the num lock, caps lock, or scroll lock indicator 
				lights on the keyboard. 
			*/

			/*
			BYTE state[256];
			GetKeyboardState(state);
			if( state[0x91] & TOGGLE_MASK ) {
				state[0x91] &= ~TOGGLE_MASK;
				SetKeyboardState(state);
			}
			*/
			return 0x6b; // F14, better than nothing

	}
	return(-1);
}

static int do_key_down(HWND hWnd, UINT uMessage, WPARAM wparam, LPARAM lparam, int alt )
{
	int key;

	if((lparam & 0x40000000) == 0) {
		key = key_win2mac(wparam,true);

		/*
		char s[100];
		wsprintf( s, "Key: 0x%X -> 0x%X\r\n", (int)wparam, (int)key );
		OutputDebugString(s);
		*/

		if(key >= 0) {
			// TODO: use interlocked functions
			if(!mac_keys_down[key]) {
				ADBKeyDown((int)key);
				mac_keys_down[key] = 1;
			}
			return(1);
		}
	}
	return(1);
}

static int do_key_up( HWND hWnd, UINT uMessage, WPARAM wparam, LPARAM lparam, int alt )
{
	int key;
	key = key_win2mac(wparam,false);
	if(key >= 0) {
		// TODO: use interlocked functions
		if(mac_keys_down[key]) {
			ADBKeyUp(key);
			mac_keys_down[key] = 0;
		}
		return(1);
	}
	return(0);
}

void suspend_emulation(void)
{
	if(CPUThread_handle && hMainWnd) {
		char caption[80];
		wsprintf( caption, "%s [slow mode]", GetString(STR_WINDOW_TITLE) );
		SetWindowText( hMainWnd, caption );
		SetThreadPriority( CPUThread_handle, THREAD_PRIORITY_BELOW_NORMAL );
	}
}

void resume_emulation(void)
{
	if(CPUThread_handle && hMainWnd) {
		SetThreadPriority( CPUThread_handle, THREAD_PRIORITY_NORMAL );
		SetWindowText( hMainWnd, GetString(STR_WINDOW_TITLE) );
	}
}

static void floppy_check(void)
{
	// TODO: should be possible to derive the parameter for SonyDiskInserted()
	// from a volume name or something. How can I now generate cd autoinsert 
	// notifications (if using emulated cdrom driver). 
	// Perhaps Sys_Open should have intnum parameter

	// Note that dsDiskInPlace etc are written from gui thread without
	// synchronization and there's a (rather remote) possibility for a conflict
	SonyDiskInserted(1);
}

static __inline__ UINT get_registered_floppy_hotkey( void )
{
	return( registered_floppy_hotkey );
}

static __inline__ bool is_shift_down(void)
{
	return( (GetAsyncKeyState(VK_SHIFT) & 0x8000) != 0 );
}

static __inline__ bool is_control_down(void)
{
	return( (GetAsyncKeyState(VK_CONTROL) & 0x8000) != 0 );
}


//////////////// window dragging ////////////////
static POINT drag_cursor;

static void drag_window_start( HWND hWnd )
{
	GetCursorPos( &drag_cursor );
}

static void drag_window_stop( HWND hWnd )
{
	RECT r;

	GetWindowRect( hWnd, &r );
	save_window_position( &r );
}

static void drag_window_move( HWND hWnd )
{
	int dx, dy;
	POINT p;
	RECT r;

	GetWindowRect( hWnd, &r );
	GetCursorPos( &p );
	dx = p.x - drag_cursor.x;
	dy = p.y - drag_cursor.y;
	drag_cursor = p;
	SetWindowPos( hWnd, 0, r.left+dx, r.top+dy, 0, 0, SWP_NOSIZE|SWP_NOZORDER );
	UpdateWindow( hWnd );
}
//////////////// window dragging ////////////////


//////////////// gui menu ////////////////
enum {
	MID_GUI_ALWAYSONTOP = 15000,
	MID_GUI_CHECKFLOPPY,
	MID_GUI_ABOUT,
	MID_GUI_MAKE_EMPTY_HFV
};

static void gui_menu( HWND hWnd )
{
  HMENU hmenu;
  POINT p;
	int flags;

	hmenu = CreatePopupMenu();

	if(win_os == VER_PLATFORM_WIN32_WINDOWS) {
		flags = MF_DISABLED|MF_GRAYED|MF_STRING;
	} else {
		flags = MF_ENABLED|MF_STRING;
	}

  AppendMenu( hmenu, MF_ENABLED|MF_STRING, MID_GUI_MAKE_EMPTY_HFV, "Make a new volume file" );
  AppendMenu( hmenu, flags, MID_GUI_CHECKFLOPPY, "Check for &Floppy\tCtrl-Shift-F11" );
  AppendMenu( hmenu, MF_ENABLED|MF_STRING, MID_GUI_ALWAYSONTOP, "Always on &Top" );
  AppendMenu( hmenu, MF_DISABLED|MF_GRAYED|MF_SEPARATOR, 0, 0  );
  AppendMenu( hmenu, MF_ENABLED | MF_STRING, MID_GUI_ABOUT, "&About..." );

	if(is_ontop) CheckMenuItem( hmenu, MID_GUI_ALWAYSONTOP, MF_CHECKED	);

	GetCursorPos( &p );
	ShowCursor(TRUE);
	tracking_menu = true;
	TrackPopupMenu( hmenu, TPM_LEFTALIGN, p.x, p.y, 0, hWnd, NULL );
	tracking_menu = false;
	ShowCursor(FALSE);
	DestroyMenu(hmenu);
	update_palette();
}
//////////////// gui menu ////////////////


static void b2_OnCommand( HWND hWnd, int id, HWND hwndCtl, UINT codeNotify )
{
	ShowCursor(TRUE);
	tracking_menu = true;

	switch(id) {
		case MID_GUI_ALWAYSONTOP:
			is_ontop = !is_ontop;
			if(is_ontop) {
				SetWindowPos( hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE );
			} else {
				SetWindowPos( hWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE );
			}
			save_ontop();
			break;
		case MID_GUI_MAKE_EMPTY_HFV:
			new_volume(hWnd);
			break;
		case MID_GUI_CHECKFLOPPY:
			floppy_check();
			break;
		case MID_GUI_ABOUT:
			about(hWnd);
			break;
	}
	ShowCursor(FALSE);
	tracking_menu = false;
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	int answer;

	// D(bug("MainWndProc 0x%X\n",(long)msg));

	switch (msg) {
		case WM_COMMAND:
			HANDLE_WM_COMMAND( hWnd, wParam, lParam, b2_OnCommand );
			return(0);
			break;

		case WM_DEVICECHANGE:
			if( wParam == DBT_DEVICEREMOVECOMPLETE ) {
				DEV_BROADCAST_HDR *p;
				p = (DEV_BROADCAST_HDR *)lParam;
				if(p->dbch_devicetype == DBT_DEVTYP_VOLUME) {
					media_removed();
				}
			} else if( wParam == DBT_DEVICEARRIVAL ) {
				DEV_BROADCAST_HDR *p;
				p = (DEV_BROADCAST_HDR *)lParam;
				if(p->dbch_devicetype == DBT_DEVTYP_VOLUME) {
					media_arrived();
				}
			}
			return DefWindowProc(hWnd, msg, wParam, lParam);

		case WM_CREATE:
			return(0);

		case WM_DESTROY:
			PostQuitMessage(0);
      return(0);

#ifdef WANT_HAVE_FAST_TASK_SWITCH
		case WM_CANCELMODE:
			if(has_own_desktop() && is_screen_inited()) {
				D(bug("cancelmode: swapping desktop\r\n"));
				swap_desktop();
				return(0);
			}
			return(DefWindowProc(hWnd, msg, wParam, lParam));
			break;
#endif

	  case WM_HOTKEY:
			if(get_registered_desktop_hotkey() == wParam) {
				swap_desktop();
			} else if(get_registered_floppy_hotkey() == wParam) {
				floppy_check();
			}
      return(0);

		case WM_KILLFOCUS:
			D(bug("WM_KILLFOCUS\r\n"));
			video_activate( false );
			int i;
			for( i=0; i<MAX_KEYS; i++ ) {
				if(mac_keys_down[i]) {
					mac_keys_down[i] = 0;
					ADBKeyUp(i);
				}
			}
			if(!has_own_desktop()) suspend_emulation();
			return(0);

		case WM_SETFOCUS:
			D(bug("WM_SETFOCUS\r\n"));
			video_activate( true );
			if(!has_own_desktop()) resume_emulation();
			//ShowWindow(hMainWnd, SW_MAXIMIZE);
			return(0);

		case WM_NCACTIVATE:
		case WM_ACTIVATEAPP:
			if((BOOL) wParam) {
				video_activate( true );
				if(!has_own_desktop()) {
					resume_emulation();
					if(is_windowed) {
						ShowWindow(hMainWnd, SW_SHOWNORMAL);
					} else {
						ShowWindow(hMainWnd, SW_MAXIMIZE);
					}
					SetActiveWindow(hMainWnd);
					SetFocus(hMainWnd);
				}
			} else {
				video_activate( false );
				int i;
				for( i=0; i<MAX_KEYS; i++ ) {
					if(mac_keys_down[i]) {
						mac_keys_down[i] = 0;
						ADBKeyUp(i);
					}
				}
				if(!has_own_desktop()) suspend_emulation();
			}
			return DefWindowProc(hWnd, msg, wParam, lParam);

		case WM_CLOSE:
			{ static bool inhere = false;
				// Yes it's possible to have many of these in a row
				if(!inhere) {
					inhere = true;
					ShowCursor(TRUE);
					video_activate( false );
					answer = MessageBox(
						hWnd, 
						"You should normally close Basilisk II only from \"Special\"/\"Shut Down\".\r\n"
						"Really close now (some data may be lost)?", 
						GetString(STR_WINDOW_TITLE), 
						MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON2
					);
					ShowCursor(FALSE);
					if(answer != IDNO) {
						QuitEmulator();
					} else {
						video_activate( true );
					}
					inhere = false;
				}
			}
			return(0);

		case WM_CHAR:
		case WM_DEADCHAR:
		case WM_SYSDEADCHAR:
			break;

		case WM_PALETTECHANGED:
			D(bug("WM_PALETTECHANGED\r\n"));
			if (hWnd != (HWND) wParam) {
				D(bug("Not our window, updating palette\r\n"));
				// video_activate( true );
				update_palette();
				return DefWindowProc(hWnd, msg, wParam, lParam);
			}
			break;
			
		case WM_QUERYNEWPALETTE:
			int retval;
			D(bug("WM_QUERYNEWPALETTE\r\n"));
			video_activate( true );
			retval = update_palette();
			return(retval);

		case WM_PAINT:
			Screen_Draw_All( hWnd );
			return(0);

		case WM_ERASEBKGND:
			// This is needed to prevent flashing
			return(1);

    case WM_SIZE:
			if(wParam == SIZE_MINIMIZED) {
				if(!has_own_desktop()) suspend_emulation();
			} else if(wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED) {
				if(!has_own_desktop()) resume_emulation();
			}
			return DefWindowProc(hWnd, msg, wParam, lParam);
			// return(0);

		case WM_KEYDOWN:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			if(wParam == get_registered_floppy_hotkey() && 
				 is_shift_down() && 
				 is_control_down())
			{
				floppy_check();
			} else if(wParam == get_registered_desktop_hotkey() && 
				 is_shift_down() && 
				 is_control_down())
			{
				// If proper hotkey mechanism fails.
				swap_desktop();
			} else {
				do_key_down(hWnd,msg,wParam,lParam,0);
			}
			return(0);

		case WM_KEYUP:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			if(wParam == get_registered_floppy_hotkey() && 
				 is_shift_down() && 
				 is_control_down()) 
			{
				// skip floppy hotkey up
			} else if(wParam == get_registered_desktop_hotkey() && 
				 is_shift_down() && 
				 is_control_down())
			{
				// If proper hotkey mechanism fails.
				// skip desktop hotkey up
			} else {
				do_key_up(hWnd,msg,wParam,lParam,0);
			}
			return(0);

		case WM_SYSKEYDOWN:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			do_key_down(hWnd,msg,wParam,lParam,1);
			return(0);

		case WM_SYSKEYUP:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			do_key_up(hWnd,msg,wParam,lParam,1);
			return(0);

		case WM_RBUTTONDOWN:
			if(!capturing && is_windowed) {
				dragging = true;
				SetCapture(hWnd);
				drag_window_start(hWnd);
			}
			break;

		case WM_RBUTTONDBLCLK:
		case WM_RBUTTONUP:
			if(is_windowed) {
				if(dragging) {
					ReleaseCapture();
					dragging = false;
					drag_window_stop(hWnd);
				}
			}
			// if(display_type == DISPLAY_WINDOW) {
				if(msg == WM_RBUTTONDBLCLK) {
					gui_menu(hWnd);
				}
			// }
			break;

		case WM_LBUTTONDOWN:
		case WM_LBUTTONDBLCLK:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			// This is atomic
			ADBMouseDown(0);
			break;

		case WM_LBUTTONUP:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			// This is atomic
			ADBMouseUp(0);
			if(capturing) {
				capturing = false;
				ReleaseCapture();
			}
			break;

		case WM_MOUSEMOVE:
			if(tracking_menu) return DefWindowProc(hWnd, msg, wParam, lParam);
			if(dragging) {
				drag_window_move(hWnd);
				return DefWindowProc(hWnd, msg, wParam, lParam);
			}

			// Better to query the position again. There may be several mouse move
			// messages in the queue, and we want to update the last position
			// as soon as possible. It is better to lose one message than show
			// a "rubber-string" effect which can drive anybody nuts

			POINT mp;
			GetCursorPos( &mp );
			ScreenToClient( hWnd, &mp );

			ADBMouseMoved( mp.x, mp.y );
			if(!dragging && !capturing) {
				if( wParam & (MK_LBUTTON|MK_MBUTTON|MK_RBUTTON) ) {
					capturing = true;
					SetCapture(hWnd);
				}
			}
			break;

		default:
			return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 1;
}

static void alloc_memory(void)
{
	RAMBaseHost = new uint8[RAMSize];
	if(!RAMBaseHost) {
		ErrorAlert(GetString(STR_NO_RAM_AREA_ERR));
		QuitEmulator();
	}

#if REAL_ADDRESSING
	RAMBaseMac = (uint32)RAMBaseHost;
#else
	RAMBaseMac = 0;
#endif

	// Create area for Mac ROM
	ROMBaseHost = new uint8[0x100000];
	if(!ROMBaseHost) {
		ErrorAlert(GetString(STR_NO_ROM_AREA_ERR));
		QuitEmulator();
	}
#if REAL_ADDRESSING
	ROMBaseMac = (uint32)ROMBaseHost;
#else
	ROMBaseMac = 0x40800000;
#endif
}

void QuitEmulator(void)
{
	D(bug("quitting...\r\n"));

	final_keyboard_hook();
	unregister_desktop_hotkey();

	// Save XPRAM
	XPRAMExit();

	// Exit video
	VideoExit();

	// Exit clipboard
	ClipExit();

	// Exit Time Manager
	TimerExit();

	// Exit serial ports
	SerialExit();

	// Exit network
	EtherExit();

	// Exit drivers
	SCSIExit();
	CDROMExit();
	DiskExit();
	SonyExit();


	// Delete ROM area
	// Delete RAM area

	if(mem_8_only) {
		// free_triple();
	} else {
		if(ROMBaseHost) delete[] ROMBaseHost;
		if(RAMBaseHost) delete[] RAMBaseHost;
	}

	// Exit system routines
	SysExit();

	// Exit preferences
	PrefsExit();

  // Let the system delete this
	// DeleteCriticalSection( &intflag_csection );

	xpar_final();

	exit(0);
}

static void initialize(void)
{
	OSVERSIONINFO osv;

	win_os = VER_PLATFORM_WIN32_WINDOWS;
	osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if(GetVersionEx( &osv )) {
		if(osv.dwPlatformId == VER_PLATFORM_WIN32s) {
			ErrorAlert("This program does not run on on Win32s.");
			QuitEmulator();
		} else {
			win_os = osv.dwPlatformId;
		}
		if(osv.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
			if(osv.dwBuildNumber <= 1000) {
				win_os_old = TRUE;
			}
		}
	}
	timer_init();

  InitializeCriticalSection( &intflag_csection );

	// Initialize variables
	RAMBaseHost = NULL;
	ROMBaseHost = NULL;

	srand(time(NULL));
	tzset();

	// Print some info
	// printf(GetString(STR_ABOUT_TEXT1), VERSION_MAJOR, VERSION_MINOR);
	// printf(" %s\n", GetString(STR_ABOUT_TEXT2));

	// Read preferences
	PrefsInit();

	// Init system routines
	SysInit();

	CPUIs68060 = false;

	// Set (fake) CPU and FPU type (UAE emulation)
	// CPU type (2 = 68020, 3 = 68030, 4 = 68040/060)
	CPUType = 2;

	// FPU type (0 = no FPU, 1 = 68881, 2 = 68882)
	FPUType = PrefsFindBool("fpu") ? 1 : 0;

	// Should usually be disabled, fpu is really not working,
	// but it seems that some 512k ROM's fail to boot w/o a FPU
	// FPUType = 0;

#define ONEMB (1024*1024)
#define FOURMB (4*1024*1024)

	// Create area for Mac RAM
	RAMSize = PrefsFindInt32("ramsize");
	if (RAMSize < ONEMB) {
		WarningAlert(GetString(STR_SMALL_RAM_WARN));
		RAMSize = ONEMB;
	}

	// Round up. Must be aligned.
	if(RAMSize % FOURMB) {
		RAMSize = (RAMSize/FOURMB + 1) * FOURMB;
	}

	if(mem_8_only) {
		// alloc_memory_8();
	} else {
		alloc_memory();
	}

	// Get rom file path from preferences
	const char *rom_path = PrefsFindString("rom");

	HFILE rom_fd = _lopen(rom_path ? rom_path : ROM_FILE_NAME, OF_READ);
	if (rom_fd < 0) {
		ErrorAlert(GetString(STR_NO_ROM_FILE_ERR));
		QuitEmulator();
	}
	ROMSize = _llseek(rom_fd, 0, SEEK_END);
	if (ROMSize != 1024*1024 && ROMSize != 512*1024) {
		ErrorAlert(GetString(STR_ROM_SIZE_ERR));
		_lclose(rom_fd);
		QuitEmulator();
	}
	_llseek(rom_fd, 0, SEEK_SET);
	if (_lread(rom_fd, ROMBaseHost, ROMSize) != ROMSize) {
		ErrorAlert(GetString(STR_ROM_FILE_READ_ERR));
		_lclose(rom_fd);
		QuitEmulator();
	}
	_lclose(rom_fd);

	// Load XPRAM
	XPRAMInit();

	// Set boot volume
	int16 i16 = PrefsFindInt16("bootdrive");
	XPRAM[0x78] = i16 >> 8;
	XPRAM[0x79] = i16 & 0xff;

	i16 = PrefsFindInt16("bootdriver");
	XPRAM[0x7a] = i16 >> 8;
	XPRAM[0x7b] = i16 & 0xff;

	if(!mem_8_only) {
		// Do not create any threads before this code
		const char *mode_str = PrefsFindString("screen");
		if(mode_str && (strncmp(mode_str, "fb",2) == 0) && (win_os == VER_PLATFORM_WIN32_NT)) {
			if(!init_desktop()) {
				ErrorAlert("Could not create new desktop for frame buffer access. Maybe not administrator rights?");
				QuitEmulator();
			}
		}
	}

	// Init drivers

	// TODO: device access should be protected by mutexes or critical sections

	SonyInit();
	DiskInit();
	CDROMInit();
	SCSIInit();

	// Init serial ports
	SerialInit();

	// Init network
	EtherInit();

	// Init Time Manager
	TimerInit();

	// Init clipboard
	ClipInit();

#if !REAL_ADDRESSING
	// Initialize UAE memory banks
	memory_init();
#endif

	// Init video
	if (!VideoInit()) {
		ErrorAlert("Failed to init video.");
		QuitEmulator();
	}

	init_keyboard_hook();
	register_desktop_hotkey();

#if !REAL_ADDRESSING
	// Reinitialize UAE memory banks (VideoInit() set the frame buffer)
	memory_init();
#endif

	// Install ROM patches
	if (!PatchROM()) {
		ErrorAlert(GetString(STR_UNSUPPORTED_ROM_TYPE_ERR));
		QuitEmulator();
	}

	if(mem_8_only) {
		// DWORD OldProtect;
    // VirtualProtect( (LPVOID)ROMBaseHost, ROMSize, PAGE_READONLY, &OldProtect );
	}


	unsigned int tid; // Temp. Don't need thread ids.

	// Start 60Hz thread
	TickThread_handle = (HANDLE)_beginthreadex( 0, 0, tick_func, 0, 0, &tid );

	// Start 1Hz thread
	OneSecThread_handle = (HANDLE)_beginthreadex( 0, 0, one_sec_func, 0, 0, &tid );

	// Start 68k and jump to ROM boot routine
	CPUThread_handle = (HANDLE)_beginthreadex( 0, 0, cpu_thread, 0, 0, &tid );

	// Start XPRAM watchdog thread
	PramThread_handle = (HANDLE)_beginthreadex( 0, 0, xpram_func, 0, 0, &tid );

	// Boost GUI operations to one notch above interrupt thread.
	// Must choose which one is more important, insanely accurate interrupts
	// or smooth mouse movement. I feel that mouse is the more important

	// Consequently, palette animation is now executed at top priority
	// which is not what I really wanted. Could be a problem in 3d shooters
	// when you get hit
	SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_HIGHEST );
}

void WarningAlert(const char *text)
{
	ShowCursor(TRUE);
	video_activate( false );
	MessageBox(hMainWnd,text,GetString(STR_WINDOW_TITLE),MB_OK|MB_ICONINFORMATION);
	video_activate( true );
	ShowCursor(FALSE);
}

void ErrorAlert(const char *text)
{
	ShowCursor(TRUE);
	video_activate( false );
	MessageBox(hMainWnd,text,GetString(STR_WINDOW_TITLE),MB_OK|MB_ICONSTOP);
	video_activate( true );
	ShowCursor(FALSE);
}

void __inline__ SetInterruptFlag(uint32 flag)
{
	EnterCriticalSection( &intflag_csection );
	InterruptFlags |= flag;
	LeaveCriticalSection( &intflag_csection );
}

void ClearInterruptFlag(uint32 flag)
{
	EnterCriticalSection( &intflag_csection );
	InterruptFlags &= ~flag;
  LeaveCriticalSection( &intflag_csection );
}

/*
 *  Code was patched, flush caches if neccessary (i.e. when using a real 680x0
 *  or a dynamically recompiling emulator)
 */

#if EMULATED_68K
void FlushCodeCache(void *start, uint32 size)
{
}
#endif

unsigned int WINAPI cpu_thread(LPVOID param)
{
	set_desktop();
	init_m68k();
	m68k_reset();
	m68k_go(true);
	_endthreadex( 0 );
	return 0;
}

/*
 *  60Hz thread
 */

unsigned int WINAPI tick_func(LPVOID param)
{
	SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL );

	for (;;) {
		Sleep(16);
		// Trigger 60Hz interrupt
		SetInterruptFlag(INTFLAG_60HZ);
		TriggerInterrupt();
	}
	_endthreadex( 0 );
	return 0;
}

/*
 *  1Hz thread
 */

unsigned int WINAPI one_sec_func(LPVOID param)
{
	DWORD mac_boot_secs = 0;
	time_t t;
  int32 should_have_ticks;

	time( &t );
	t = t - timezone + (daylight ? 3600 : 0) + TIME_OFFSET;
	mac_boot_secs = t - GetTickCount()/1000 ;
	WriteMacInt32(0x20c, t);

	should_have_ticks = GetTickCount() + 1000;

	// Pseudo Mac 1Hz interrupt, update local time
	for (;;) {
		Sleep( should_have_ticks - (int)GetTickCount() );
		WriteMacInt32(0x20c, mac_boot_secs + GetTickCount()/1000);
		should_have_ticks += 1000;
	}
	_endthreadex( 0 );
	return 0;
}

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd )
{
	// Most of the code below deals with detecting the previous
	// instance. We cannot afford having two copies running.
	// The perceived complexity of the code arises from the fact
	// that we may have created a separate desktop object 
	// and the windows in there are not readily enumerable
	// without switching desktop first.

	// This mutant is never acquired, used only for run detection
	const char *mname = "Basilisk II is running";

	// FindWindow() works only within a given desktop. 
	// Need to use a global method.
	HANDLE mutant = OpenMutex( MUTEX_ALL_ACCESS, FALSE, mname );
	if(mutant) {
		CloseHandle(mutant);

		HDESK hDesk = OpenDesktop( get_desktopname(), 0, FALSE, GENERIC_ALL );
		if(hDesk) {
			USEROBJECTFLAGS uof;
			uof.fInherit = FALSE;
			uof.fReserved = FALSE;
			uof.dwFlags = DF_ALLOWOTHERACCOUNTHOOK;
			SetUserObjectInformation( hDesk, UOI_FLAGS, (LPVOID)&uof, sizeof(uof) );
			SetThreadDesktop(hDesk);
			SwitchDesktop(hDesk);
		}

		// Now it's safe to use FindWindow().
		// Search by class since title varies.
		HWND w = FindWindow( get_wnd_class_name(), 0 );
		if(w) {
			PostMessage( w, WM_HOTKEY,
				(WPARAM)get_registered_desktop_hotkey(), 0 );
		}
		return(0);
	}
	mutant = CreateMutex( FALSE, FALSE, mname );

	hInst = hInstance;

	if(*lpCmdLine) {
		SetPrefsFile(lpCmdLine);
	} else {
		SetPrefsFile("BasiliskII_prefs");
	}

	GUIThread_handle = GetCurrentThread();

	initialize();

	MSG msg;
	while ( GetMessage(&msg, NULL, 0, 0) ) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	if(mutant) CloseHandle(mutant);
	return(msg.wParam);
}
