/*
 *  prefs_windows.cpp - Preferences handling for Win32
 *
 *  Basilisk II (C) 1997-1999 Christian Bauer
 *
 *  Ported to Windows by Lauri Pesonen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>

#include "sysdeps.h"
#include "prefs.h"
#include "prefs_windows.h"


// Platform-specific preferences items
prefs_desc platform_prefs_items[] = {
	{"replacescsi", TYPE_STRING, true},		// replacescsi "HP" "CD-Writer+ 7100" "PHILIPS" "CDD3600"
	{"disablescsi", TYPE_STRING, true},		// disablescsi "TOSHIBA" "CD-ROM XM-6202B"
	{"nofloppyboot", TYPE_BOOLEAN, false},
	{"noscsi", TYPE_BOOLEAN, false},
	{"ntdx5hack", TYPE_BOOLEAN, false},
	{NULL, TYPE_END, false}	// End of list
};


// Prefs file name and path
static char prefs_path[_MAX_PATH];


static void prefs_get_ini_path( char *path, int sz )
{
	memset( path, 0, sz );
	strncpy( path, prefs_path, sz-1 );
}

void SetPrefsFile( char *path )
{
	memset( prefs_path, 0, sizeof(prefs_path) );

	if(!strchr(path,'\\')) {
		GetCurrentDirectory(sizeof(prefs_path)-1,prefs_path);
		if(strlen(prefs_path) > 0 && prefs_path[strlen(prefs_path)-1] != '\\') strcat(prefs_path,"\\");
	}
	strcat( prefs_path, path );
}

/*
 *  Load preferences from settings file
 */

void LoadPrefs(void)
{
	char path[_MAX_PATH];

	prefs_get_ini_path(path,sizeof(path));

	// Read preferences from settings file
	FILE *f = fopen(path, "r");
	if (f != NULL) {
		// Prefs file found, load settings
		LoadPrefsFromStream(f);
		fclose(f);
	} else {
		// No prefs file, save defaults
		SavePrefs();
	}
}


/*
 *  Save preferences to settings file
 */

void SavePrefs(void)
{
	char path[_MAX_PATH];
	FILE *f;

	prefs_get_ini_path(path,sizeof(path));

	if ((f = fopen(path, "w")) != NULL) {
		SavePrefsToStream(f);
		fclose(f);
	}
}
