/*
 *  util_windows.cpp - Miscellaneous utilities for Win32
 *
 *  Basilisk II (C) 1997-1999 Christian Bauer
 *
 *  Ported to Windows by Lauri Pesonen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "sysdeps.h"
#include "util_windows.h"

// These are not time critical, used only when emulation is firing up
// Needed to avoid linking against winsock library

uint32 b2_ntohl( uint32 s )
{
	return(
		 ((s&0x000000FF) << 24) |
     ((s&0x0000FF00) << 8 ) |
     ((s&0x00FF0000) >> 8 ) |
     ((s&0xFF000000) >> 24)
	);
}

uint32 b2_htonl( uint32 s )
{
	return(
		 ((s&0x000000FF) << 24) |
     ((s&0x0000FF00) << 8 ) |
     ((s&0x00FF0000) >> 8 ) |
     ((s&0xFF000000) >> 24)
	);
}

uint16 b2_ntohs( uint16 s )
{
	return( ((s&0x00FF) << 8) | ((s&0xFF00) >> 8) );
}

uint16 b2_htons( uint16 s )
{
	return( ((s&0x00FF) << 8) | ((s&0xFF00) >> 8) );
}

bool exists( char *path )
{
	HFILE h;
	bool ret = false;

	h = _lopen( path, OF_READ );
	if(h != HFILE_ERROR) {
		ret = true;
		_lclose(h);
	}
	return(ret);
}

bool create_file( char *path, DWORD size )
{
	HANDLE h;
	bool ok = false;

	h = CreateFile( path,
		GENERIC_READ | GENERIC_WRITE,
		0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL
	);
	if(h != INVALID_HANDLE_VALUE) {
		if(SetFilePointer( h, size, NULL, FILE_BEGIN) != 0xFFFFFFFF) {
			if(SetEndOfFile(h)) {
				ok = true;
			}
		}
		CloseHandle(h);
	}
	if(!ok) DeleteFile(path);
	return(ok);
}
