/*
 *  xpram_windows.cpp - XPRAM handling for Win32
 *
 *  Basilisk II (C) 1997-1999 Christian Bauer
 *
 *  Ported to Windows by Lauri Pesonen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>

#include "sysdeps.h"
#include "xpram.h"
#include "xpram_windows.h"
#include "desktop_windows.h"


// XPRAM file name and path
const char XPRAM_FILE_NAME[] = "pram.dat";
static char param_path[_MAX_PATH];

static HANDLE prefs_file_mutex = 0;

#define XPRAM_SIZE (sizeof(XPRAM))

static uint8 last_xpram[XPRAM_SIZE];

static bool xpar_grab_mutex()
{
	if(!prefs_file_mutex) {
		prefs_file_mutex = CreateMutex( 0, FALSE, "Basilisk II prefs file mutex" );
	}
	if(prefs_file_mutex) {
		WaitForSingleObject(prefs_file_mutex,INFINITE);
		return(true);
	} else {
		return(false);
	}
}

static void xpar_release_mutex()
{
	if(prefs_file_mutex) {
		ReleaseMutex(prefs_file_mutex);
	}
}

// Should call CloseHandle(prefs_file_mutex) but Windows does this anyway at exit


/*
 *  XPRAM watchdog thread (saves XPRAM every minute)
 */

unsigned int WINAPI xpram_func(LPVOID param)
{
	set_desktop();

	memcpy(last_xpram, XPRAM, XPRAM_SIZE);

	SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL );

	for (;;) {
		Sleep(60000);
		if (memcmp(last_xpram, XPRAM, XPRAM_SIZE)) {
			memcpy(last_xpram, XPRAM, XPRAM_SIZE);
			SaveXPRAM();
		}
	}
	_endthreadex( 0 );
	return(0);
}

static void xpar_get_ini_path( char *path, int sz )
{
	if(!*param_path) {
		GetCurrentDirectory(sz,param_path);
		if(strlen(param_path) > 0 && param_path[strlen(param_path)-1] != '\\') strcat(param_path,"\\");
		strcat( param_path, XPRAM_FILE_NAME );
	}
	memset( path, 0, sz );
	strncpy( path, param_path, sz );
}

/*
 *  Load XPRAM from settings file
 */

void LoadXPRAM(void)
{
	HFILE hf;
	char path[_MAX_PATH];

	if(xpar_grab_mutex()) {
		xpar_get_ini_path(path,sizeof(path));
		hf = _lopen( path, OF_READ );
		if(hf != HFILE_ERROR) {
			_lread(hf, XPRAM, XPRAM_SIZE);
			_lclose(hf);
		}
		xpar_release_mutex();
	}
}


/*
 *  Save XPRAM to settings file
 */

void SaveXPRAM(void)
{
	HFILE hf;
	char path[_MAX_PATH];

	if(xpar_grab_mutex()) {
		xpar_get_ini_path(path,sizeof(path));
		unlink( path );
		hf = _lcreat( path, 0 );
		if(hf != HFILE_ERROR) {
			_lwrite(hf, (char *)XPRAM, XPRAM_SIZE);
			_lclose(hf);
		}
		xpar_release_mutex();
	}
}

void xpar_final(void)
{
	if(prefs_file_mutex) {
		CloseHandle(prefs_file_mutex);
		prefs_file_mutex = 0;
	}
}

